
 TITLE CBM64 Emulator by Paul TOTH (tothpaul@mygale.org)

comment *
Copyright (C) 1996-98  Paul Toth (tothpaul@mygale.org)

http://www.mygale.org/~tothpaul

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*

; Define DEBUG (use MAKE.BAT or DEBUG.BAT)

.MODEL Tiny
.CODE
.386

StartupCode

JMP BEGIN

;--- DATA ---

CopyRight:
 db 'C=64 Emulator for 386+ by Paul TOTH <tothpaul@mygale.org>',13,10
 db 'Copyright (C) MySoft - ',??DATE,13,10
 db 'http://www.mygale.org/~tothpaul',13,10,'$'

digits db '0123456789ABCDEF'

TotalCycles dd 0

msgExit    db 'Exit at $'
msgCRLF    db 13,10,'$'
msgCycles  db ' cycles $'
msgSeconds db ' seconds.',13,10,'$'
msgRate    db 'Rate : $'
msgPercent db '%$'

OutOfMemory:
 db 'Sorry, need 3*64K (196.608) of memory$'

 Ticks dd 0

INCLUDE DATA.ASM

;--- Memory Selectors ---

Align 2
RAMSeg dw 1000h ; = CS + 1000h
ROMSeg dw 2000h ; = CS + 2000h
CIOSeg dw 0000h ; = CS
VGASeg dw 0A000h
LOWSeg dw 0040h

;--- CBM64 Registers ---

 A DW 0
 X DW 0
 Y DW 0
 S DW 1FFh
 P DB 0

Align 2
 PC DW 0FCE2h

 Cycles DW 64

;--- Flags
 CFlag equ 1
 ZFlag equ 2
 IFlag equ 4
 DFlag equ 8
 BFlag equ 16
 VFlag equ 64
 NFlag equ 128

;--- Sub Routines

INCLUDE PEEK.ASM
INCLUDE POKE.ASM
INCLUDE 6510.ASM
INCLUDE VIC2.ASM
INCLUDE CIA1.ASM
INCLUDE KEYB.ASM
INCLUDE KEYB.INC

;--- Main Program ---

BEGIN:

;--- Setup Memory ---
IF DEBUG
 cmp byte ptr cs:[0],0EAh ; far jump
 je  DBG64
  mov dx,Offset DBG64NotFound
  mov ah,09h
  int 21h
 ret
 DBG64NotFound: db 'To start this debug version, you need to run DBG64.EXE !$'
 DBG64:

ELSE

 MOV AH,4Ah
 MOV BX,3000h ; allocate 3*64K (CS, RAM, ROM)
 INT 21h
 JNC Allocated
  mov dx,Offset CopyRight
  mov ah,09h
  int 21h
  mov dx,Offset OutOfMemory
  mov ah,09h
  int 21h
  ret
 Allocated:

ENDIF

;--- Init Memory ---

 MOV AX,CS
 ADD RAMSeg,AX
 ADD ROMSeg,AX
 ADD CIOSeg,AX

 MOV SI,offset C64ROM
 MOV ES,ROMSeg

 MOV DI,0A000h	; BASIC ROM
 MOV CX,8192/2
 REP MOVSW

 MOV DI,0D000h	; CHARS ROM & KERNAL ROM
 MOV CX,(4096+8192)/2
 REP MOVSW

 MOV ES,CIOSeg  ; Clear CHIPS
 MOV DI,0D000h
 XOR AX,AX
 MOV CX,4096/2
 REP STOSW

;-- Patch Kernal (Frodo)
 MOV ES,ROMSeg
; send byte from $95 on serial bus
 MOV WORD PTR ES:[0ED40h],00F2h
; send Listen (Talk/Unlisten/Untalk) on serial bus
 MOV WORD PTR ES:[0ED23h],01F2h
; send byte from $95 on serial bus
 MOV WORD PTR ES:[0ED36h],02F2h
; Input byte on serial bus
 MOV WORD PTR ES:[0EE13h],03F2h
; Only needed for real IEC
 MOV BYTE PTR ES:[0EDEFh],60h
 MOV BYTE PTR ES:[0EDBEh],60h
 MOV BYTE PTR ES:[0EDCCh],60h
 MOV BYTE PTR ES:[0EE03h],60h

;-- Init Video --- Dr.Bob Tweak Mode 360x240
IF DEBUG
 MOV AX,5
 MOV DI,Offset A
 INT 64h
ELSE

 MOV AX,13h
 INT 10h

 MOV DX,03C4h ; Sequencer Register
 MOV AX,0604h ; Disable Chain 4 Mode
 OUT DX,AX

 MOV AX,0100h ; (A)synchronous Rset
 OUT DX,AX

 MOV DX,03C2h ; VGA Misc Register
 MOV AL,0E7h  ; Use 28Mhz Clock & 60Hz
 OUT DX,AL

 MOV DX,03C4h ; Sequencer Register
 MOV AX,0300h ; Restart Sequencer
 OUT DX,AX

 ; Diasable Write protect For CrtC Registers 0-7, since we are
 ; about to change the horizontal & vertical timing settings.

 MOV DX,03D4h ; VHA CrtC Register
 MOV AL,11h   ; CrtC Register 11h
 OUT DX,AL    ; Load current value
 INC DX       ; Point to Data
 IN  AL,DX    ; Get CrtC register 11h
 AND AL,7Fh   ; Mask out Write Protect
 OUT DX,AL    ; and send it back

 ; Send CrtC data in TweakMode array to the CrtC.
 MOV DX,03D4h ; VGA CrtC Register
 CLD          ; Forward Block Load
 MOV SI,offset TweakMode ; Get parameter data
 MOV CX,17    ; Number of entries in block
 REP OUTSW

;--- Clear Screen ---

 MOV DX,03C4h
 MOV AX,0F02h
 OUT DX,AX
 MOV ES,VGASeg
 XOR AX,AX
 XOR DI,DI
 MOV CX,360*(240/4)*3
 REP STOSW

;--- setup colors ---

 MOV DX,03C8h
 MOV AL,0
 OUT DX,AL
 INC DX
 MOV SI,offset Palette
 MOV CX,3*16
 REP OUTSB

;-- Install Keyboard
 Call InstallKeyboard

ENDIF

;-- Get Ticks
 MOV ES,LOWSeg
 MOV EAX,ES:[6Ch]
 MOV Ticks,EAX

;-- Fetch OpCode --
 MOV ES,VGASeg
 MOV FS,RAMSeg
 MOV GS,ROMSeg

 XOR CX,CX

Align 2
Next:
 SUB Cycles,CX
 JNS SkipVic

 ADD TotalCycles,64 ; ??

 Call DrawLine ;-- ! +64 / +24

 Call CheckKeyboard

;--- check CIA
 Call CIA1Line

 TEST CIA1,2 ;--- NMI
 JZ TestIRQ

  AND CIA1,255-2
 ; PUSH PC
  MOV BX,S
  MOV AX,PC
  DEC BX
  MOV FS:[BX],AX
  DEC BX
 ; PUSH P
  MOV AL,P
  MOV FS:[BX],AL
  DEC BX
  MOV S,BX
 ; IFlag
  OR P,IFlag
 ; PC=DEEK($FFFA)
  MOV BX,0FFFAh
  Call DeekBX
  MOV PC,BX
  SUB Cycles,7

 IF DEBUG
  MOV AX,6
  MOV DI,Offset A
  INT 64h
 ENDIF

 JMP SkipVic

 TestIRQ:
 TEST CIA1,1 ;--- IRQ
 JZ SkipVic

 TEST P,IFlag
 JNZ SkipVic

  AND CIA1,255-1
 ; PUSH PC
  MOV BX,S
  MOV AX,PC
  DEC BX
  MOV FS:[BX],AX
  DEC BX
 ; PUSH P
  MOV AL,P
  MOV FS:[BX],AL
  DEC BX
  MOV S,BX
 ; IFlag
  OR P,IFlag
 ; PC=DEEK($FFFE)
  MOV BX,0FFFEh
  Call DeekBX
  MOV PC,BX
  SUB Cycles,7

 IF DEBUG
  MOV AX,7
  MOV DI,Offset A
  INT 64h
 ENDIF

Align 2
 SkipVic:

ExitPatch:
 JMP PeekPC ;-- goto OpTable[OpCode] (6510.ASM)

;--- Text Mode ---

JAM:
Done:

;--- Consumme Ticks
 MOV ES,LOWSeg
 MOV EAX,ES:[6Ch]
 SUB EAX,Ticks
 MOV EBX,10
 MUL EBX
 MOV EBX,182 ; 18.2 ...
 DIV EBX
 MOV Ticks,EAX

IF DEBUG
ELSE
 Call RemoveKeyboard

 MOV AX,03h
 INT 10h

 MOV DX,Offset CopyRight
 MOV AH,09h
 INT 21h

;-- Display ExitPoint
 MOV DX,Offset msgExit
 MOV AH,09h
 INT 21h
 XOR EAX,EAX
 MOV AX,PC
 MOV EBX,16
 Call Value
 MOV DX,Offset msgCRLF
 MOV AH,09h
 INT 21h

;-- Display TotalCycles
 MOV EAX,TotalCycles
 MOV EBX,10
 Call Value

 MOV DX,Offset msgCycles
 MOV AH,09h
 INT 21h

;-- Display Ticks
 MOV EAX,Ticks
 MOV EBX,10
 Call Value

 MOV DX,Offset msgSeconds
 MOV AH,09h
 INT 21h

;-- Display %
 MOV DX,Offset msgRate
 MOV AH,09h
 INT 21h

 MOV EAX,TotalCycles
 MOV EBX,Ticks
 XOR EDX,EDX
 DIV EBX
 XOR EDX,EDX
 MOV EBX,10000
 DIV EBX
 MOV EBX,10
 Call Value

 MOV DX,Offset msgPercent
 MOV AH,09h
 INT 21h
 RET

 Value:
  XOR CX,CX
 NextValue:
  INC CX
  XOR EDX,EDX
  DIV EBX
  PUSH DX
  OR  EAX,EAX
 JNZ NextValue
 MOV AH,02h
 Print:
  POP BX
  MOV DL,[BX+Offset Digits]
  INT 21h
 Loop Print

ENDIF

RET

;--- ROMs ---

C64ROM: ;>>>>>>> COPY CBM64.COM /B + C64.ROM /B CBM64.COM <<<<<<<<<

END
