;--- CIA1 Emulation
comment *
Copyright (C) 1996-98  Paul Toth (tothpaul@mygale.org)

http://www.mygale.org/~tothpaul

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
Align 2
Start1	dw 0  ; 1 = TimerA , 2 = TimerB , 3 = Chain Timers
TimerA1 dw 0
TimerB1 dw 0

CIA1  dw 0    ; 1 = IRQ
Mask1 db 0

Align 2
KeyMatrix db 8 dup (0FFh)
RevMatrix db 8 dup (0FFh)

Align 2
CIA1Line:

 MOV CX,Cycles

 IF DEBUG
  MOV AX,9
  MOV BX,Start1
  MOV BH,Mask1
  MOV DX,TimerA1
  INT 64h
 ENDIF

;-- Timer A
 TEST Start1,1
 JZ SkipTimerA1

  SUB TimerA1,CX     ; Timer A
  JNS SkipTimerA1

  MOV AX,DS:[0DC04h] ; Reset Timer
  MOV TimerA1,AX

  MOV  AL,DS:[0DC0Eh] ; Control Register A
  TEST AL,8
  JZ ContinueA1
   AND AL,0FEh
   MOV DS:[0DC0Eh],AL
   AND Start1,2+4
  ContinueA1:

  OR byte ptr DS:[0DC0Dh],1   ; IRQ Control Register
  TEST Mask1,1
  JZ NoIRQA
   OR byte ptr DS:[0DC0Dh],80h
   OR CIA1,1
  NoIRQA:

  TEST Start1,4
  JZ SkipTimerA1

  MOV CX,1
  JMP DoTimerB1

 SkipTimerA1:

;-- Timer B
 TEST Start1,2
 JZ SkipTimerB1

 DoTimerB1:

  SUB TimerB1,CX
  JNS SkipTimerB1

  MOV AX,DS:[0DC06h]
  MOV TimerB1,AX

  MOV  AL,DS:[0DC0Fh] ; Control Register
  TEST AL,8
  JZ ContinueB1
   AND AL,0FEh
   MOV DS:[0DC0Fh],AL
   AND Start1,1
  ContinueB1:

  OR byte ptr DS:[0DC0Dh],2   ; IRQ Control Register
  TEST Mask1,2
  JZ NoIRQB
   OR byte ptr DS:[0DC0Dh],80h
   OR CIA1,1
  NoIRQB:

 SkipTimerB1:

RET

;--- Poke($D000)

Align 2
 PokeCIA1:
     ; DC00 DC01 DC02 DC03 DC04 DC05 DC06 DC07
  dw   wC1x,wC1x,wC1x,wC1x,wC1x,wC15,wC1x,wC1x
     ; DC08 DC09 DC0A DC0B DC0C DC0D DC0E DC0F
  dw   wC1x,wC1x,wC1x,wC1x,wC1x,wC1D,wC1E,wC1F

Align 2
wC1x:
 MOV [DI],AL
JMP Next

Align 2
wC15:
 MOV [DI],AL
 TEST Start1,1
 JNZ SkipC15
  MOV AX,DS:[0DC04h]
  MOV TimerA1,AX
 SkipC15:
JMP Next

Align 2
wC16:
 MOV [DI],AL
 TEST Start1,2
 JNZ SkipC16
  MOV AX,DS:[0DC06h]
  MOV TimerB1,AX
 SkipC16:
JMP Next

Align 2
wC1D:
 TEST AL,128
 JNZ OrMask
  NOT AL
  AND AL,Mask1
  JMP DoMask
 OrMask:
  AND AL,7Fh
  OR  AL,Mask1
 DoMask:
 MOV Mask1,AL
 MOV [DI],AL

 IF DEBUG
  MOV AX,10
  MOV BL,Mask1
  INT 64h
 ENDIF

JMP Next

Align 2
wC1E:
 TEST AL,10h
 JZ NotLoadA1
  MOV BX,DS:[0DC04h]
  MOV TimerA1,BX
 NotLoadA1:
 MOV AH,AL
 AND AL,0EFh
 MOV [DI],AL
 AND AH,21h
 AND Start1,255-1
 CMP AH,1
 JNE NotStartA1
  OR Start1,1
 NotStartA1:

 IF DEBUG
  MOV AX,8
  MOV BX,Start1
  INT 64h
 ENDIF

JMP Next

Align 2
wC1F:
 TEST AL,10h
 JZ NotLoadB1
  MOV BX,DS:[0DC06h]
  MOV TimerB1,AX
 NotLoadB1:
 MOV AH,AL
 AND AL,0EFh
 MOV [DI],AL
 AND AL,61h
 AND Start1,255-2-4
 CMP AH,1
 JNE NotStartB1
  OR Start1,2
 NotStartB1:
 CMP AH,41h
 JNE NotChain
  OR Start1,4
 NotChain:

 IF DEBUG
  MOV AX,8
  MOV BX,Start1
  INT 64h
 ENDIF

JMP Next

;--- Peek($D000)

Align 2
 PeekCIA1:
     ; DC00 DC01 DC02 DC03 DC04 DC05 DC06 DC07
  dw   rC10,rC11,rC1x,rC1x,rC1x,rC1x,rC1x,rC1x
     ; DC08 DC09 DC0A DC0B DC0C DC0D DC0E DC0F
  dw   rC1x,rC1x,rC1x,rC1x,rC1x,rC1D,rC1x,rC1x

Align 2
rC1x:
 MOV AL,[DI]
RET

Align 2
rC10:
 MOV AX,[DI]   ; DPA+DPB
 MOV BX,[DI+2] ; DDA+DDB
 NOT BX
 OR  AX,BX
 MOV BX,Offset RevMatrix
 MOV CX,8
 Rev1:
 SHR AH,1
 JC SkipRev1
  AND AL,[BX]
 SkipRev1:
 INC BX
 LOOP Rev1
RET

Align 2
rC11:
 MOV AX,DS:[0DC02h] ; DDA+DDB
 NOT AX
 OR  AL,DS:[0DC00h] ; DPA
 XCHG AL,AH
 MOV BX,Offset KeyMatrix
 MOV CX,8
 Key1:
 SHR AH,1
 JC SkipKey1
  AND AL,[BX]
 SkipKey1:
 INC BX
 LOOP Key1
 MOV AH,DS:[0DC01h]   ; DPB
 AND AH,DS:[0DC03h] ; DDB
 OR  AL,AH
 XOR AH,AH ; !!!

 IF DEBUG
  PUSH AX
  PUSH BX
  MOV BX,AX
  MOV AX,2
  INT 64h
  POP  BX
  POP  AX
 ENDIF

RET

Align 2
rC1D:
 MOV AL,[DI]
 MOV BYTE PTR [DI],0
RET

