
;-- Keyboard Emulation
Align 2
KeyCount  dw 0
KeyStart  dw 0
KeyBuffer db 256 dup(0) ; 0..255
Align 2
KeyState  db 0

OldInt09  dd 0

Align 2
UpdateKeyboard:
; no C=64 key
 CMP AH,0FFh
 JE  KeyDone
; C=64 Shift

 TEST AH,80h
 JZ SetupColRow

  AND AH,7Fh
  TEST AL,80h
  JNZ ReleaseShift
   AND [KeyMatrix+1],01111111b ; not (1 shl 7)
   AND [RevMatrix+7],11111101b ; not (1 shl 1)
   JMP SetupColRow
  ReleaseShift:
   OR  [KeyMatrix+1],10000000b ; (1 shl 7)
   OR  [RevMatrix+7],00000010b ; (1 shl 1)

  SetupColRow:
   MOV CH,AH
   MOV CL,CH
   SHR CH,3 ; col
   AND CL,7 ; row

   MOV BX,0101h
   SHL BL,CL
  XCHG CL,CH
   SHL BH,CL

   TEST AL,80h
   MOV AX,BX

   JNZ ReleaseKey
    XOR BH,BH
    MOV BL,CL
    NOT AX
    AND [BX+Offset KeyMatrix],AL
    MOV BL,CH
    AND [BX+Offset RevMatrix],AH
   JMP KeyDone
   ReleaseKey:
    XOR BH,BH
    MOV BL,CL
    OR  [BX+Offset KeyMatrix],AL
    MOV BL,CH
    OR  [BX+Offset RevMatrix],AH
   RET

Align 2
CheckKeyboard:
 CLI
 MOV BX,KeyStart
 CMP BX,KeyCount
 JNE ProcessKey
 STI
RET

Align 2
ProcessKey:
 INC BL
 MOV AL,[BX+Offset KeyBuffer]
 MOV KeyStart,BX
 STI

 MOV BX,AX
 AND BX,7Fh

 TEST KeyState,1 ; Extended
 JZ StandardKey

  AND KeyState,255-1

  CMP BL,2Ah
  JE  SetXShift
  CMP BL,38h
  JE  SetAltGr

  TEST KeyState,2 ; XShift
  JZ SetXKey
   MOV AH,[BX+Offset xShft-1]
   JMP UpdateKeyboard
  SetXKey:
   MOV AH,[BX+Offset xKeys-1]
  JMP UpdateKeyboard

  SetXShift:
   AND KeyState,255-2
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,2
  RET

  SetAltGr:
   AND KeyState,255-4
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,4
  RET

 StandardKey:

  CMP BL,2Ah
  JE  SetLShift
  CMP BL,36h
  JE  SetRShift
  CMP AL,0E0h  ; !!
  JE  SetXState
  CMP BL,0E1h
  JE  KeyDone

  TEST KeyState,8+16
  JZ NoShift
   MOV AH,[BX+Offset Shift-1]
   JMP UpdateKeyboard
  NoShift:
  TEST KeyState,4
  JZ NoAltGr
   MOV AH,[BX+Offset AltGr-1]
   JMP UpdateKeyboard
  NoAltGr:
   MOV AH,[BX+Offset Keybd-1]
  JMP UpdateKeyboard

  SetLShift:
   AND KeyState,255-8
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,8
  RET

  SetRShift:
   AND KeyState,255-16
   TEST AL,80h
   JNZ KeyDone
   OR KeyState,16
  RET

  SetXState:
   OR KeyState,1

 KeyDone:

 RET

Align 2
KeyboardHandler:
 PUSH AX
 PUSH BX
 PUSH DX
 PUSH DS

 PUSH CS
 POP  DS

 MOV  DX,60h ; Scan:=Port[$60];
 IN   AL,DX
 MOV  BL,AL

 INC  DX
 IN   AL,DX  ; dummy:=Port[$61];
 MOV  AH,AL
 OR   AL,80h ; Port[$61]:=dummy or $80;
 OUT  DX,AL
 MOV  AL,AH  ; Port[$61]:=dummy;
 OUT  DX,AL

 MOV  AL,BL

 MOV  BX,KeyCount
 INC  BL
 CMP  BX,KeyStart
 JE ToManyKeys
  MOV [BX+Offset KeyBuffer],AL
  MOV KeyCount,BX
 ToManyKeys:

 SHR AL,1
 JNZ NotEscKey
  MOV word ptr ExitPatch,01EBh ; JMP+1
 NotEscKey:

 MOV DX,20h
 MOV AL,DL
 OUT DX,AL

 POP DS
 POP DX
 POP BX
 POP AX
IRET

InstallKeyboard:

  mov ax,3509h ; Keyboard INT
  int 21h
  mov word ptr OldInt09,bx
  mov word ptr (OldInt09+2),es

  mov dx,offset KeyboardHandler
  mov ax,2509h
  int 21h

RET

RemoveKeyboard:
 push ds
  lds dx,OldInt09
  mov ax,2509h
  int 21h
 pop ds
RET

